import argparse

from slack_transfer.run import main_run
from slack_transfer.run import set_parser_run
from slack_transfer.version import __version__

from .bookmark import main_bookmark
from .bookmark import set_parser_bookmark
from .download import main_download
from .download import set_parser_download
from .emoji import main_emoji
from .emoji import set_parser_emoji
from .file_volume import main_file_volume
from .file_volume import set_parser_file_volume
from .token_test import main_token_test
from .token_test import set_parser_token_test
from .upload import main_upload
from .upload import set_parser_upload


def main() -> None:
    print(
        f"""=========================
Welcome to slack_transfer
This is CLI v.{__version__}
========================="""
    )
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    parser_run = subparsers.add_parser("run", help="see `run -h`")
    set_parser_run(parser=parser_run)
    parser_run.set_defaults(handler=main_run)

    parser_file_volume = subparsers.add_parser(
        "file_volume", help="see `file_volume -h`"
    )
    set_parser_file_volume(parser=parser_file_volume)
    parser_file_volume.set_defaults(handler=main_file_volume)

    parser_download = subparsers.add_parser("download", help="see `download -h`")
    set_parser_download(parser=parser_download)
    parser_download.set_defaults(handler=main_download)

    parser_upload = subparsers.add_parser("upload", help="see `upload -h`")
    set_parser_upload(parser=parser_upload)
    parser_upload.set_defaults(handler=main_upload)

    parser_bookmark = subparsers.add_parser("bookmark", help="see `bookmark -h`")
    set_parser_bookmark(parser=parser_bookmark)
    parser_bookmark.set_defaults(handler=main_bookmark)

    parser_emoji = subparsers.add_parser("emoji", help="see `emoji -h`")
    set_parser_emoji(parser=parser_emoji)
    parser_emoji.set_defaults(handler=main_emoji)

    parser_token_test = subparsers.add_parser("token_test", help="see `token_test -h`")
    set_parser_token_test(parser=parser_token_test)
    parser_token_test.set_defaults(handler=main_token_test)

    try:
        import tkinter

        from slack_transfer.interactive import interactive

        interactive_wrap = lambda _: interactive()
        parser_interactive = subparsers.add_parser(
            "interactive", help="interactive mode."
        )
        parser_interactive.set_defaults(handler=interactive_wrap)
    except:
        pass

    args = parser.parse_args()
    if hasattr(args, "handler"):
        args.handler(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



import datetime
import json
import os.path
import time
import warnings
from typing import Callable
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union
from unittest.mock import patch

import tqdm
from dateutil import tz
from markdownify import markdownify
from slack_sdk.errors import SlackApiError
from slack_sdk.web import SlackResponse

from .._base import UploaderClientABC
from .common import get_channels_list


def create_all_channels(
    client: UploaderClientABC,
    channel_names: Optional[List[str]] = None,
    name_mappings: Optional[Dict[str, str]] = None,
) -> None:
    """create all channels or specified channels. In this process, channel purpose and description are also set.

    Args:
        client (UploaderClientABC): uploader client. If use this via any UploaderClient Class, self is automatically set.
            Thus, ignore this.
        channel_names (List[str]; Optional; default=None): list of channels to be made.
        name_mappings (Dict[str, str]; Optional; default=None): You can set name mappings between the channel names of the
            original and destination workspaces. For example, :code:`{"old_name1": "new_name1", "old_name2": "new_name2"}`.
    """
    downloaded_channels = json.load(
        open(
            os.path.join(client.local_data_dir, "channels.json"),
            mode="r",
            encoding="utf-8",
        )
    )
    if channel_names:
        downloaded_channels = list(
            filter(lambda x: (x["name"] in channel_names), downloaded_channels)  # type: ignore
        )
    else:
        downloaded_channels = list(
            filter(
                lambda x: os.path.exists(
                    os.path.join(client.local_data_dir, "channels", f"{x['name']}.json")
                ),
                downloaded_channels,
            )
        )
    channels_list: List[Dict] = get_channels_list(client=client)
    for old_channel_info in downloaded_channels:
        new_channel_name = (
            name_mappings[old_channel_info["name"]]
            if name_mappings and old_channel_info["name"] in name_mappings
            else old_channel_info["name"]
        )
        new_channel_if_exists = list(
            filter(lambda x: x["name"] == new_channel_name, channels_list)
        )
        if len(new_channel_if_exists) != 0:
            if len(new_channel_if_exists) > 1:
                raise AssertionError(
                    f"Error: multiple channels exists ({new_channel_name})"
                )
            continue
        response = client.conversations_create(name=new_channel_name)
        if not response["ok"]:
            raise IOError(f"Error in making channel {new_channel_name}")
        new_channel_info = response["channel"]
        new_channel_id = new_channel_info["id"]
        client.conversations_setTopic(
            channel=new_channel_id, topic=old_channel_info["topic"]["value"]
        )
        client.conversations_setPurpose(
            channel=new_channel_id, purpose=old_channel_info["purpose"]["value"]
        )


class HTTP_send_output:
    def __init__(
        self,
        allow_slower: bool = True,
        progress: bool = False,
        chunk_size: int = 200 * 1024,
    ) -> None:
        self.allow_slower = allow_slower
        self.progress = progress
        self.chunk_size = chunk_size

    def _send_output(self, cls, message_body=None, encode_chunked=False):  # type: ignore
        """Send the currently buffered request and clear the buffer.

        Appends an extra \\r\\n to the buffer.
        A message_body may be specified, to be appended to the request.
        """
        cls._buffer.extend((b"", b""))
        msg = b"\r\n".join(cls._buffer)
        del cls._buffer[:]
        cls.send(msg)

        if message_body is not None:

            # create a consistent interface to message_body
            if hasattr(message_body, "read"):
                # Let file-like take precedence over byte-like.  This
                # is needed to allow the current position of mmap'ed
                # files to be taken into account.
                chunks = cls._read_readable(message_body)
            else:
                try:
                    # this is solely to check to see if message_body
                    # implements the buffer API.  it /would/ be easier
                    # to capture if PyObject_CheckBuffer was exposed
                    # to Python.
                    memoryview(message_body)
                except TypeError:
                    try:
                        chunks = iter(message_body)
                    except TypeError:
                        raise TypeError(
                            "message_body should be a bytes-like "
                            "object or an iterable, got %r" % type(message_body)
                        )
                else:
                    # the object implements the buffer interface and
                    # can be passed directly into socket methods
                    chunks = (message_body,)

            for chunk in chunks:
                if not chunk:
                    if cls.debuglevel > 0:
                        print("Zero length chunk ignored")
                    continue

                if len(chunk) < 8 * 1024 * 1024 or not self.allow_slower:
                    if encode_chunked and cls._http_vsn == 11:
                        # chunked encoding
                        chunk = f"{len(chunk):X}\r\n".encode("ascii") + chunk + b"\r\n"
                    cls.send(chunk)
                else:
                    print("Now using slower uploader backend")
                    chunk_size = self.chunk_size
                    n_chunks = int((len(chunk) - 1) // chunk_size) + 1
                    last_time = time.time()
                    for i in tqdm.tqdm(range(n_chunks), disable=not self.progress):
                        sub_chunk = chunk[
                            (i * chunk_size) : min(len(chunk), (i + 1) * chunk_size)
                        ]
                        if encode_chunked and cls._http_vsn == 11:
                            # chunked encoding
                            sub_chunk = (
                                f"{len(sub_chunk):X}\r\n".encode("ascii")
                                + sub_chunk
                                + b"\r\n"
                            )
                        cls.send(sub_chunk)
                        sec_from_last = time.time() - last_time
                        time.sleep(max(0.0, 1.0 / 5 - sec_from_last))
                        last_time = time.time()

            if encode_chunked and cls._http_vsn == 11:
                # end chunked transfer
                cls.send(b"0\r\n\r\n")

    def return_function(self) -> Callable:
        def fn_send_output(cls, message_body=None, encode_chunked=False):  # type: ignore
            self._send_output(
                cls=cls, message_body=message_body, encode_chunked=encode_chunked
            )

        return fn_send_output


HTTP_BACKEND = HTTP_send_output(allow_slower=False, progress=False)


def upload_file(
    client: UploaderClientABC,
    old_file_id: str,
    file_name: str,
    channel_id: Optional[str] = None,
    title: Optional[str] = None,
    filetype: Optional[str] = None,
    is_slack_post: bool = False,
    thread_ts: Optional[str] = None,
) -> Optional[Tuple[str, str]]:
    """upload a file. This is usually preprocess for posting message attaching files.
    **So, this method is not usually directly used by users.**

    Args:
        client (UploaderClientABC): uploader client. If use this via any UploaderClient Class, self is automatically set.
            Thus, ignore this.
        old_file_id (str): old file id. This is required to find the file in local data dir.
        file_name (str): file name. It have to be the original old file name.
            This is also required to find the file in local data dir. The uploaded file is also named as the same.
        channel_id (str; Optional; default=None): channel id (usually 9-digits string)
        title (str; Optional; default=None): title for appending to the file when uploading.
        filetype (str; Optional; default=None): file type. It is not mimetype. See: https://api.slack.com/types/file#types
        is_slack_post (bool; Optional; default=False): set True if this file is slack post.
        thread_ts (str; Optional; default=None): thread ts.
    """
    file_path: Optional[str] = os.path.join(
        client.local_data_dir, "files", f"{old_file_id}--{file_name}"
    )
    content = None
    if is_slack_post:
        filetype = "post"
        if file_path is None:
            raise AssertionError
        # ToDo: better support of slack post
        content = (
            markdownify(
                json.load(open(file_path, mode="r"))["full"]
                .replace("<pre>", "```")
                .replace("</pre>", "```")
                .replace("<p>", "")
                .replace("</p>", "")
            )
            .replace("\n\n\n", "\n")
            .replace("\n\n", "\n")
        )
        file_path = None
    upload_start_st = int(time.time())
    upload_results: Tuple[str, str]
    delete_targets: List[str]
    for i in range(3):
        try:
            with patch(
                "http.client.HTTPConnection._send_output",
                HTTP_BACKEND.return_function(),
            ):
                response: SlackResponse = client.files_upload(
                    file=file_path,
                    content=content,
                    filename=file_name,
                    filetype=filetype,
                    title=title,
                    channels=channel_id,
                    thread_ts=thread_ts,
                )
            if not response["ok"]:
                raise IOError(f"Error in uploading file {file_path}")
            return response["file"]["id"], response["file"]["permalink"]
        except FileNotFoundError as e:
            print(
                f"file is missing (possibly duu to original slack limitations): {file_path}"
            )
            return None
        except Exception as e:
            print(e)
            # ToDo: workaround for #11
            n_try = 5
            for i_try in range(n_try):
                try:
                    print(
                        f"uploaded file is waiting to be processed. Wait {30*i_try} sec."
                    )
                    time.sleep(30 * i_try)
                    _response = client.files_list(
                        channel=channel_id, ts_from=str(upload_start_st)
                    )
                    # ToDo: paging: たぶんほとんどの場合不要
                    if not _response["ok"] or len(_response["files"]) == 0:
                        raise IOError(f"Error in uploading file {file_name}")
                    file_candidates = list(
                        filter(
                            lambda x: x["name"] == file_name and x["title"] == title,
                            _response["files"],
                        )
                    )
                    if len(file_candidates) == 0:
                        raise IOError(f"Error in uploading file {file_name}")
                    return (file_candidates[0]["id"], file_candidates[0]["permalink"])
                except Exception as e:
                    pass
        if HTTP_BACKEND.allow_slower is False:
            HTTP_BACKEND.allow_slower = True
        else:
            if HTTP_BACKEND.chunk_size > 50 * 1024:
                HTTP_BACKEND.chunk_size = int(HTTP_BACKEND.chunk_size // 2.0)
            else:
                break

    raise IOError(f"Error in uploading file {file_name}")


def check_channel_exists(client: UploaderClientABC, channel_name: str) -> bool:
    """check if a channel already exists or not

    Args:
        client (UploaderClientABC): uploader client. If use this via any UploaderClient Class, self is automatically set.
            Thus, ignore this.
        channel_name (str): channel name wanted to be checked.

    Yield:
         bool: return True if exists. Otherwise, return False.
    """
    channels_list: List[Dict] = get_channels_list(client=client)
    new_channel_infos = list(filter(lambda x: x["name"] == channel_name, channels_list))
    if len(new_channel_infos) > 1:
        raise AssertionError
    return len(new_channel_infos) == 1


def check_insert_finished(
    client: UploaderClientABC,
    channel_name: str,
    old_members_dict: Dict[str, str],
    old_channel_name: Optional[str] = None,
    time_zone: str = "Asia/Tokyo",
) -> bool:
    """check if the uploading for a channel already finished or not. This check is rough. Just checking some latest messages.
    Note that it raise error if the channel doesn't exist.

    Args:
        client (UploaderClientABC): uploader client. If use this via any UploaderClient Class, self is automatically set.
            Thus, ignore this.
        channel_name (str): channel name wanted to be checked.
        old_members_dict (Dict[str, str]): a dictionary between old user id and old user preview name.
            (Old means the original WS) key is 9-digit string if and value is the preview user name.
        old_channel_name (str; Optional; default=None): old channel name. It required to find the stored data.
        time_zone (str; Optional; default=Asia/Tokyo): time zone to preview the original post data on the destination WS.
            See: https://dateutil.readthedocs.io/en/stable/tz.html

    Yield:
         bool: return True if finished. Otherwise, return False.
    """
    # return error when channel not exists
    tz_delta = tz.gettz(time_zone)
    channels_list: List[Dict] = get_channels_list(client=client)
    new_channel_infos = list(filter(lambda x: x["name"] == channel_name, channels_list))
    if len(new_channel_infos) != 1:
        raise AssertionError("channel possibly does not exists")
    new_channel_info = new_channel_infos[0]
    new_channel_id = new_channel_info["id"]
    data_file_name = (
        f"{old_channel_name}.json" if old_channel_name else f"{channel_name}.json"
    )
    data_file_path = os.path.join(client.local_data_dir, "channels", data_file_name)
    old_messages: List[Dict] = json.load(
        open(data_file_path, mode="r", encoding="utf-8")
    )
    old_messages = list(
        filter(
            lambda x: "thread_ts" not in x
            or ("subtype" in x and x["subtype"] == "thread_broadcast"),
            old_messages,
        )
    )
    old_messages.sort(key=lambda x: x["ts"], reverse=False)

    response = client.conversations_history(channel=new_channel_id)
    new_messages = response["messages"]
    new_messages.sort(key=lambda x: x["ts"], reverse=False)

    expected_username_dict: Dict[str, int] = {}
    for i in range(1, min(len(old_messages) + 1, 10)):
        expected_date_time = datetime.datetime.fromtimestamp(
            float(old_messages[-i]["ts"]), tz=tz_delta
        ).strftime("%Y/%m/%d %H:%M %Z")
        expected: str = (
            (
                old_members_dict[old_messages[-i]["user"]]
                if "user" in old_messages[-i]
                and old_messages[-i]["user"] in old_members_dict
                else "Unknown member"
            )
            + " ["
            + expected_date_time
            + "]"
        )

        if expected in expected_username_dict:
            expected_username_dict[expected] += 1
        else:
            expected_username_dict[expected] = 1
    for key, value in expected_username_dict.items():
        if (
            len(
                list(
                    filter(
                        lambda x: "username" in x and x["username"] == key, new_messages
                    )
                )
            )
            < value
        ):
            return False
    return True


def data_insert(
    client: UploaderClientABC,
    channel_name: str,
    old_members_dict: Dict[str, str],
    old_members_icon_url_dict: Optional[Dict[str, str]] = None,
    old_channel_name: Optional[str] = None,
    time_zone: str = "Asia/Tokyo",
    progress: Union[bool, tqdm.tqdm] = True,
) -> str:
    """upload messages, files, reactions, and pins.

    Args:
        client (UploaderClientABC): uploader client. If use this via any UploaderClient Class, self is automatically set.
            Thus, ignore this.
        channel_name (str): channel name wanted to be checked.
        old_members_dict (Dict[str, str]): a dictionary between old user id and old user preview name.
            (Old means the original WS) key is 9-digit string if and value is the preview user name.
        old_members_icon_url_dict (Dict[str, str]]; Optional; default=None): a dictionary between old user id and old user
            icon url. key is 9-digit string if and value is url.
        old_channel_name (str; Optional; default=None): old channel name. It required to find the stored data.
        time_zone (str; Optional; default=Asia/Tokyo): time zone to preview the original post data on the destination WS.
            See: https://dateutil.readthedocs.io/en/stable/tz.html
        progress (Union[bool, tqdm.tqdm]; Optional; default=True): set progress bar. progress bar will be updated each time one
            message is processed. You can set progress bar from outside this method if you want to make the whole progress bar
            beyond each progress of each channel. If you set it to boolean, it is understood as if you want to show a progress
            bar only for this processing.

    Yield:
         str: channel id (9-digit string)
    """
    if old_members_icon_url_dict is None:
        old_members_icon_url_dict = {}
    tz_delta = tz.gettz(time_zone)
    if tz_delta is None:
        tz_delta = datetime.timezone(datetime.timedelta(hours=9), name="JST")
    channels_list: List[Dict] = get_channels_list(client=client)
    new_channel_info = list(filter(lambda x: x["name"] == channel_name, channels_list))[
        0
    ]
    new_channel_id = new_channel_info["id"]

    data_file_name = (
        f"{old_channel_name}.json" if old_channel_name else f"{channel_name}.json"
    )
    data_file_path = os.path.join(client.local_data_dir, "channels", data_file_name)
    messages: List[Dict] = json.load(open(data_file_path, mode="r", encoding="utf-8"))
    messages.sort(key=lambda x: x["ts"], reverse=False)
    ts_mapper: Dict[str, str] = {}
    progress_bar: tqdm.tqdm
    if isinstance(progress, bool):
        progress_bar = tqdm.tqdm(total=len(messages), disable=not progress)
    else:
        progress_bar = progress
    response: SlackResponse = client.chat_postMessage(
        channel=new_channel_id,
        text="File thread for message migration by slack_transfer. Please ignore this.",
    )
    file_thread_ts = response["message"]["ts"]
    for message in messages:
        file_ids = []
        file_permalinks = []
        file_titles = []
        if "files" in message:
            for file in message["files"]:
                if "name" in file and "title" in file and "filetype" in file:
                    old_file_id = file["id"]
                    file_name = file["name"]
                    title = file["title"]
                    file_type = file["filetype"]
                    upload_results = None
                    try:
                        upload_results = upload_file(
                            client=client,
                            old_file_id=old_file_id,
                            file_name=file_name,
                            channel_id=new_channel_id,
                            title=title,
                            filetype=file_type,
                            is_slack_post=(
                                file["mimetype"] == "application/vnd.slack-docs"
                                if "mimetype" in file
                                else False
                            ),
                            thread_ts=file_thread_ts,
                        )
                    except:
                        with open(
                            os.path.join(
                                client.local_data_dir, "file_upload_failure.txt"
                            ),
                            mode="a",
                            encoding="utf-8",
                        ) as f:
                            f.write(
                                f"{datetime.datetime.now().strftime('%Y/%m/%d %H:%M:%S')}, {file_name} ({old_file_id}--{file_name})\n"
                            )

                    if upload_results:
                        new_file_id, new_file_permalink = upload_results
                        file_ids.append(new_file_id)
                        file_permalinks.append(new_file_permalink)
                        file_titles.append(title)

        new_thread_ts = None
        if "thread_ts" in message.keys() and message["thread_ts"] != "":
            if message["thread_ts"] in ts_mapper:
                new_thread_ts = ts_mapper[message["thread_ts"]]

        i_count = 0
        while True:
            try:
                date_time = datetime.datetime.fromtimestamp(
                    float(message["ts"]), tz=tz_delta
                ).strftime("%Y/%m/%d %H:%M %Z")
                blocks: List[Dict] = (
                    list(
                        filter(
                            lambda x: x["type"] not in ["rich_text", "call"],
                            message["blocks"],
                        )
                    )
                    if "blocks" in message
                    else []
                ) + [
                    {"type": "file", "source": "remote", "file_id": file_id}
                    for file_id in file_ids
                ]

                if len(blocks) != 0:
                    blocks = (
                        [
                            {
                                "type": "section",
                                "text": {
                                    "type": "mrkdwn",
                                    "text": message["text"][
                                        i
                                        * 3000 : min(
                                            len(message["text"]), (i + 1) * 3000
                                        )
                                    ],
                                },
                            }
                            for i in range((len(message["text"]) - 1) // 3000 + 1)
                        ]
                        if message["text"]
                        else []
                    ) + blocks

                if (
                    len(blocks) == 0
                    and message["text"] == ""
                    and "attachments" not in message
                ):
                    message[
                        "text"
                    ] = "[Migration Error] this thread is missing (possibly due to original slack limitation)"
                attachments: List[Dict] = (
                    message["attachments"] if "attachments" in message else []
                )
                for i_attachment in range(len(attachments)):
                    if "blocks" in attachments[i_attachment]:
                        attachments[i_attachment]["blocks"] = list(
                            filter(
                                lambda x: x["type"] not in ["call"],
                                attachments[i_attachment]["blocks"],
                            )
                        )
                response = client.chat_postMessage(
                    channel=new_channel_id,
                    text=message["text"],
                    attachments=attachments,
                    blocks=blocks,
                    thread_ts=new_thread_ts,
                    reply_broadcast=(
                        message["subtype"] == "thread_broadcast"
                        if "subtype" in message
                        else False
                    ),
                    username=(
                        old_members_dict[message["user"]]
                        if "user" in message and message["user"] in old_members_dict
                        else "Unknown member"
                    )
                    + " ["
                    + date_time
                    + "]",
                    icon_url=old_members_icon_url_dict[message["user"]]
                    if "user" in message
                    and message["user"] in old_members_icon_url_dict
                    else None,
                )
                break
            except SlackApiError as e:
                if e.response["error"] in ["ratelimited", "fatal_error"]:
                    i_count += 1
                    if i_count < 10:
                        time.sleep(2)
                        continue
                raise e

        if not response["ok"]:
            raise IOError(f"Error in posting message {message['text']}")

        if "reactions" in message:
            for reaction in message["reactions"]:
                try:
                    client.reactions_add(
                        channel=new_channel_id,
                        name=reaction["name"],
                        timestamp=response["ts"],
                    )
                except SlackApiError as e:
                    if e.response["error"] == "invalid_name":
                        warnings.warn(
                            f"reaction `{reaction['name']}` doesn't exist. Skip adding this reaction."
                        )
                    else:
                        raise e

        if "pinned_to" in message:
            channels_ids_pined = message["pinned_to"]
            if len(channels_ids_pined) > 1:
                # ToDo: "pined to multiple channels is not supported"
                raise NotImplemented
            client.pins_add(channel=new_channel_id, timestamp=response["ts"])

        if "thread_ts" in message.keys() and message["thread_ts"] != "":
            if message["thread_ts"] in ts_mapper:
                pass
            else:
                ts_mapper[message["thread_ts"]] = response["message"]["ts"]
        else:
            ts_mapper[message["ts"]] = response["message"]["ts"]
        progress_bar.update(n=1)
    return new_channel_id


def check_upload_conflict(
    client: UploaderClientABC, name_mappings: Optional[Dict[str, str]] = None
) -> List[str]:
    """check upload conflict in terms of channel name.

    This is not considered if the channel is already finished processing. Just checking name conflict.

    Args:
        client (UploaderClientABC): uploader client. If use this via any UploaderClient Class, self is automatically set.
            Thus, ignore this.
        name_mappings (Dict[str, str]; Optional; default=None): You can set name mappings between the channel names of the
            original and destination workspaces. For example, :code:`{"old_name1": "new_name1", "old_name2": "new_name2"}`.

    Yield:
        List[str]: name conflicting channels' names.
    """
    existing_channels = get_channels_list(client=client)
    downloaded_channels = json.load(
        open(
            os.path.join(client.local_data_dir, "channels.json"),
            mode="r",
            encoding="utf-8",
        )
    )
    downloaded_channels = list(
        filter(
            lambda x: os.path.exists(
                os.path.join(client.local_data_dir, "channels", f"{x['name']}.json")
            ),
            downloaded_channels,
        )
    )
    existing_channels_name = set(map(lambda x: x["name"], existing_channels))
    downloaded_channels_name = set(
        map(
            lambda x: name_mappings[x["name"]]
            if name_mappings is not None and x["name"] in name_mappings
            else x["name"],
            downloaded_channels,
        )
    )
    return list(existing_channels_name & downloaded_channels_name)


def insert_bookmarks(
    client: UploaderClientABC, channel_id: str, old_channel_name: str = None
) -> None:
    """insert a series of bookmarks to a channel.

    This method is not checking if the bookmarks are already inserted or not.

    Args:
        client (UploaderClientABC): uploader client. If use this via any UploaderClient Class, self is automatically set.
            Thus, ignore this.
        channel_id (str; Optional; default=None): channel id (usually 9-digits string)
        old_channel_name (str; Optional; default=None): old channel name. It required to find the stored data.
    """
    data_file_path = os.path.join(
        client.local_data_dir, "bookmarks", f"{old_channel_name}.json"
    )
    if not os.path.exists(data_file_path):
        return
    bookmarks: List[Dict] = json.load(open(data_file_path, mode="r", encoding="utf-8"))
    for bookmark in bookmarks:
        del bookmark["channel_id"]
        client.bookmarks_add(channel_id=channel_id, **bookmark)



import argparse

from slack_transfer.run import run


def set_parser_download(parser: argparse.ArgumentParser) -> None:
    """See download section in :doc:`../../../user_guide/cli`"""
    parser.add_argument(
        "--data_dir",
        "-d",
        type=str,
        required=True,
        help="Data directory for saving download data or loading upload data. This is required.",
    )
    parser.add_argument(
        "--downloader_token",
        "-td",
        type=str,
        required=True,
        help="Download token obtained from slack (the original workspace). Starts with `xoxb-`",
    )
    parser.add_argument(
        "--channel_names",
        type=str,
        default=None,
        help="channel names you want to download. If not set, set to all available channels. "
        + "Set by comma-separation for multiple inputs. For example, `general,random`",
    )
    parser.add_argument(
        "--skip_bookmarks", action="store_true", help="Skip process bookmarks."
    )


def main_download(args: argparse.Namespace) -> None:
    """See download section in :doc:`../../../user_guide/cli`"""
    if args.channel_names is not None:
        channel_names = args.channel_names.split(",")
    else:
        channel_names = None

    run(
        local_data_dir=args.data_dir,
        downloader_token=args.downloader_token,
        skip_upload=True,
        channel_names=channel_names,
        skip_bookmarks=args.skip_bookmarks,
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    set_parser_download(parser=parser)
    args = parser.parse_args()
    main_download(args=args)



import argparse

from slack_transfer.run import run


def set_parser_upload(parser: argparse.ArgumentParser) -> None:
    """See upload section in :doc:`../../../user_guide/cli`"""
    parser.add_argument(
        "--data_dir",
        "-d",
        type=str,
        required=True,
        help="Data directory for saving upload data or loading upload data. This is required.",
    )
    parser.add_argument(
        "--uploader_token",
        "-tu",
        type=str,
        required=True,
        help="upload token obtained from slack (the destination workspace). Starts with `xoxb-`",
    )
    parser.add_argument(
        "--old_channel_names",
        type=str,
        default=None,
        help="channel names you want to upload. Name is selected among downloaded WS. "
        + "If not set, set to all available channels. "
        + "Set by comma-separation for multiple inputs. For example, `general,random`",
    )
    parser.add_argument(
        "--override",
        action="store_true",
        help="This flag enables message migration to the destination workspace even when a channel already exists."
        + " This means that additional messages are inserted after the message already sent to the channel."
        + " If you want not to do so, please delete the channel on the destination workspace first.",
    )
    parser.add_argument(
        "--name_mappings",
        type=str,
        default=None,
        help="You can set name mappings between the channel names of the original and destination workspaces. "
        + "Comma-separated dictionaries (key:value) are available. For example, `old_name1:new_name1,old_name2:new_name2`.",
    )
    parser.add_argument(
        "--skip_bookmarks", action="store_true", help="Skip process bookmarks."
    )


def main_upload(args: argparse.Namespace) -> None:
    """See upload section in :doc:`../../../user_guide/cli`"""
    if args.name_mappings:
        name_mappings = dict(
            [
                (dict_input.split(":")[0], dict_input.split(":")[1])
                for dict_input in args.name_mappings.split(",")
            ]
        )
    else:
        name_mappings = None

    if args.old_channel_names is not None:
        channel_names = args.old_channel_names.split(",")
    else:
        channel_names = None

    run(
        local_data_dir=args.data_dir,
        uploader_token=args.uploader_token,
        override=args.override,
        skip_download=True,
        name_mappings=name_mappings,
        channel_names=channel_names,
        skip_bookmarks=args.skip_bookmarks,
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    set_parser_upload(parser=parser)
    args = parser.parse_args()
    main_upload(args=args)



import argparse
import glob
import json
import os.path
import time
from ssl import SSLContext
from typing import Dict
from typing import List
from typing import Optional

import tqdm

from .client import DownloaderClient
from .client import UploaderClient


def run(
    local_data_dir: str,
    downloader_token: Optional[str] = None,
    uploader_token: Optional[str] = None,
    override: bool = False,
    skip_download: bool = False,
    skip_upload: bool = False,
    name_mappings: Optional[Dict[str, str]] = None,
    channel_names: Optional[List[str]] = None,
    skip_bookmarks: bool = False,
    ssl: Optional[SSLContext] = None,
    time_zone: str = "Asia/Tokyo",
) -> None:
    """See run section in :doc:`/user_guide/cli`

    Args:
        local_data_dir (str): Data directory for saving download data or loading upload data. This is required.
        downloader_token (str; Optional; default=None): Download token obtained from slack (the original workspace).
            Starts with xoxb-. This is required when skip_download=False.
        uploader_token (str; Optional; default=None): Upload token obtained from slack (the destination workspace).
            Starts with xoxb-. This is required when skip_upload=False.
        override (bool; Optional; default=False): This flag enables message migration to the destination workspace
            even when a channel already exists. This means that additional messages are inserted after the message
            already sent to the channel. If you want not to do so, please delete the channel on the destination workspace first.
        skip_download (bool; Optional; default=False): Skip download. This is usually used when the download is already finished.
        skip_upload (bool; Optional; default=False): Skip upload. This is usually used when only the download is necessary.
        name_mappings (Dict[str, str]; Optional; default=None): You can set name mappings between the channel names of the
            original and destination workspaces. For example, :code:`{"old_name1": "new_name1", "old_name2": "new_name2"}`.
        channel_names (List[str]; Optional; default=None): channel names you want to process.
            If not set, set to all available channels.
        skip_bookmarks (bool; Optional; default=False): Skip process bookmarks.
        ssl: (SSLContext; Optional; default=None): : An [`ssl.SSLContext`][1] instance, helpful for specifying
            your own custom certificate chain.
        time_zone (str; Optional; default=Asia/Tokyo): time zone to preview the original post data on the destination WS.
            See: https://dateutil.readthedocs.io/en/stable/tz.html
    """
    os.makedirs(local_data_dir, exist_ok=True)
    if not skip_download:
        downloader = DownloaderClient(
            local_data_dir=local_data_dir, token=downloader_token, ssl=ssl
        )
        downloader.test_connection()
        downloader.test_downloader()
    if not skip_upload:
        if uploader_token is None:
            raise ValueError("uploader_token is required")
        uploader = UploaderClient(
            local_data_dir=local_data_dir, token=uploader_token, ssl=ssl
        )
        uploader.test_uploader()
    if not skip_download:
        channels_list: List[Dict] = downloader.download_channels_list()
        downloader.download_members_list()

        if channel_names:
            channels_list = list(
                filter(lambda x: (x["name"] in channel_names), channels_list)  # type: ignore
            )

        ts_now = int(time.time())
        times_to_rest = list(map(lambda x: ts_now - x["created"], channels_list))
        for i, (channel, time_to_rest) in enumerate(zip(channels_list, times_to_rest)):
            print(f"{i+1}/{len(channels_list)}: {channel['name']}")
            pbar = tqdm.tqdm(total=time_to_rest)
            downloader.download_channel_history(
                channel_id=channel["id"],
                channel_name=channel["name"],
                ts_progress_bar=pbar,
                ts_now=ts_now,
            )
            pbar.close()
            time.sleep(1)
            if not skip_bookmarks:
                downloader.download_bookmark(
                    channel_id=channel["id"], channel_name=channel["name"]
                )

    if not skip_upload:
        if name_mappings is None:
            name_mappings = {}

        old_members = json.load(
            open(
                os.path.join(uploader.local_data_dir, "members.json"),
                mode="r",
                encoding="utf-8",
            )
        )
        old_members_dict = dict(
            [(member["id"], member["profile"]["real_name"]) for member in old_members]
        )
        old_members_icon_url_dict = dict(
            [
                (
                    member["id"],
                    member["profile"]["image_original"]
                    if "image_original" in member["profile"]
                    else member["profile"]["image_512"],
                )
                for member in old_members
            ]
        )

        conflicts = uploader.check_upload_conflict(name_mappings=name_mappings)
        reverse_name_mappings = dict([(v, k) for k, v in name_mappings.items()])
        conflicts = list(
            filter(
                lambda x: (  # type: ignore
                    (
                        x
                        in [
                            name_mappings[y] if y in name_mappings else y
                            for y in channel_names
                        ]
                    )
                    if channel_names
                    else True
                    and uploader.check_channel_exists(channel_name=x)
                    and not uploader.check_insert_finished(
                        channel_name=x,
                        old_members_dict=old_members_dict,
                        old_channel_name=(
                            reverse_name_mappings[x]
                            if x in reverse_name_mappings
                            else x
                        ),
                        time_zone=time_zone,
                    )
                ),
                conflicts,
            )
        )
        if len(conflicts) > 0 and not override:
            raise ValueError(
                f"channels: {', '.join(conflicts)} are already exist. please set mapping or override=True"
            )

        uploader.create_all_channels(
            channel_names=channel_names, name_mappings=name_mappings
        )

        channel_files: List[str] = glob.glob(
            os.path.join(uploader.local_data_dir, "channels", "*.json")
        )
        if channel_names:
            channel_files = list(
                filter(
                    lambda x: (os.path.basename(x)[:-5] in channel_names),  # type: ignore
                    channel_files,
                )
            )
        for i, channel_file_path in enumerate(channel_files):
            old_channel_name = os.path.basename(channel_file_path).replace(".json", "")
            if old_channel_name in name_mappings:
                new_channel_name = name_mappings[old_channel_name]
            else:
                new_channel_name = old_channel_name
            print(
                f"{i + 1}/{len(channel_files)}: {old_channel_name} -> {new_channel_name}"
            )
            if (
                uploader.check_channel_exists(channel_name=new_channel_name)
                and uploader.check_insert_finished(
                    channel_name=new_channel_name,
                    old_members_dict=old_members_dict,
                    old_channel_name=old_channel_name,
                    time_zone=time_zone,
                )
                and not override
            ):
                print("already finished. skip.")
                continue
            new_channel_id = uploader.data_insert(
                channel_name=new_channel_name,
                old_members_dict=old_members_dict,
                old_members_icon_url_dict=old_members_icon_url_dict,
                old_channel_name=old_channel_name,
                time_zone=time_zone,
            )
            if not skip_bookmarks:
                uploader.insert_bookmarks(
                    channel_id=new_channel_id, old_channel_name=old_channel_name
                )


def set_parser_run(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "--data_dir",
        "-d",
        type=str,
        required=True,
        help="Data directory for saving download data or loading upload data. This is required.",
    )
    parser.add_argument(
        "--downloader_token",
        "-td",
        type=str,
        default=None,
        help="Download token obtained from slack (the original workspace). Starts with `xoxb-`",
    )
    parser.add_argument(
        "--uploader_token",
        "-tu",
        type=str,
        default=None,
        help="Upload token obtained from slack (the destination workspace). Starts with `xoxb-`",
    )
    parser.add_argument(
        "--override",
        action="store_true",
        help="This flag enables message migration to the destination workspace even when a channel already exists."
        + " This means that additional messages are inserted after the message already sent to the channel."
        + " If you want not to do so, please delete the channel on the destination workspace first.",
    )
    parser.add_argument(
        "--skip_download",
        action="store_true",
        help="Skip download. This is usually used when the download is already finished.",
    )
    parser.add_argument(
        "--skip_upload",
        action="store_true",
        help="Skip upload. This is usually used when only the download is necessary.",
    )
    parser.add_argument(
        "--channel_names",
        type=str,
        default=None,
        help="channel names you want to process. If not set, set to all available channels. "
        + "Set by comma-separation for multiple inputs. For example, `general,random`",
    )
    parser.add_argument(
        "--name_mappings",
        type=str,
        default=None,
        help="You can set name mappings between the channel names of the original and destination workspaces. "
        + "Comma-separated dictionaries (key:value) are available. For example, `old_name1:new_name1,old_name2:new_name2`.",
    )
    parser.add_argument(
        "--skip_bookmarks", action="store_true", help="Skip process bookmarks."
    )


def main_run(args: argparse.Namespace) -> None:
    name_mappings = None
    if args.name_mappings:
        name_mappings = dict(
            [
                (dict_input.split(":")[0], dict_input.split(":")[1])
                for dict_input in args.name_mappings.split(",")
            ]
        )
    if args.channel_names is not None:
        channel_names = args.channel_names.split(",")
    else:
        channel_names = None
    if name_mappings:
        print(f"name mappings: {name_mappings}")
    run(
        local_data_dir=args.data_dir,
        downloader_token=args.downloader_token,
        uploader_token=args.uploader_token,
        override=args.override,
        skip_download=args.skip_download,
        skip_upload=args.skip_upload,
        channel_names=channel_names,
        name_mappings=name_mappings,
        skip_bookmarks=args.skip_bookmarks,
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    set_parser_run(parser=parser)
    args = parser.parse_args()
    main_run(args=args)



import logging
import os
from ssl import SSLContext
from typing import Dict
from typing import List
from typing import Optional
from typing import Union

from slack_sdk import WebClient
from slack_sdk.http_retry import RetryHandler

from .functions.common import get_channels_list
from .functions.common import get_file_volumes
from .functions.common import get_replies
from .functions.common import test_connection
from .functions.common import test_downloader
from .functions.common import test_uploader


class CommonDryRunClient(WebClient):
    """This class is generated to run dry-run operation.
    Inherit :code:`slack_sdk.WebClient`.

    .. seealso::
        - https://github.com/slackapi/python-slack-sdk

    Args:
        token (str; Optional): A string specifying an `xoxp-*` or `xoxb-*` token.
        base_url (str; Optional): A string representing the Slack API base URL.
            Default is `'https://www.slack.com/api/'`
        timeout (int; Optional): The maximum number of seconds the client will wait
            to connect and receive a response from Slack.
            Default is 30 seconds.
        ssl (SSLContext; Optional): An [`ssl.SSLContext`][1] instance, helpful for specifying
            your own custom certificate chain.
        proxy (str; Optional): String representing a fully-qualified URL to a proxy through
            which to route all requests to the Slack API. Even if this parameter
            is not specified, if any of the following environment variables are
            present, they will be loaded into this parameter: `HTTPS_PROXY`,
            `https_proxy`, `HTTP_PROXY` or `http_proxy`.
        headers (dict; Optional): Additional request headers to attach to all requests.

    .. seealso::
        - :code:`CommonDryRunClient.get_channels_list`: :func:`slack_transfer.functions.common.get_channels_list`
        - :code:`CommonDryRunClient.get_replies`: :func:`slack_transfer.functions.common.get_replies`
        - :code:`CommonDryRunClient.get_file_volumes`: :func:`slack_transfer.functions.common.get_file_volumes`
    """

    def get_channels_list(self) -> List[Dict]:
        """get channel list :func:`slack_transfer.functions.common.get_channels_list`"""
        return get_channels_list(client=self)

    def get_replies(self, channel_id: str, ts: str) -> List[Dict]:
        """get replies for a message :func:`slack_transfer.functions.common.get_replies`"""
        return get_replies(client=self, channel_id=channel_id, ts=ts)

    def get_file_volumes(
        self, channel_ids: Optional[Union[str, List[str]]] = None
    ) -> float:
        """calculate rough volume for files which will be downloaded (Dry-run mode) :func:`slack_transfer.functions.common.get_file_volumes`"""
        return get_file_volumes(client=self, channel_ids=channel_ids, auto_join=False)


class CommonNoLocalVolumeClient(CommonDryRunClient):
    """This class is generated to run without local volume. (Some small modification can be caused on WS.)
    Inherit :code:`slack_sdk.WebClient` via :func:`slack_transfer._base.CommonDryRunClient`

    .. seealso::
        - https://github.com/slackapi/python-slack-sdk

    Args:
        token (str; Optional): A string specifying an `xoxp-*` or `xoxb-*` token.
        base_url (str; Optional): A string representing the Slack API base URL.
            Default is `'https://www.slack.com/api/'`
        timeout (int; Optional): The maximum number of seconds the client will wait
            to connect and receive a response from Slack.
            Default is 30 seconds.
        ssl (SSLContext; Optional): An [`ssl.SSLContext`][1] instance, helpful for specifying
            your own custom certificate chain.
        proxy (str; Optional): String representing a fully-qualified URL to a proxy through
            which to route all requests to the Slack API. Even if this parameter
            is not specified, if any of the following environment variables are
            present, they will be loaded into this parameter: `HTTPS_PROXY`,
            `https_proxy`, `HTTP_PROXY` or `http_proxy`.
        headers (dict; Optional): Additional request headers to attach to all requests.

    .. seealso::
        - :code:`CommonNoLocalVolumeClient.get_channels_list`: :func:`slack_transfer.functions.common.get_channels_list`
        - :code:`CommonNoLocalVolumeClient.get_replies`: :func:`slack_transfer.functions.common.get_replies`
        - :code:`CommonNoLocalVolumeClient.get_file_volumes`: :func:`slack_transfer.functions.common.get_file_volumes`
        - :code:`CommonNoLocalVolumeClient.test_connection`: :func:`slack_transfer.functions.common.test_connection`
        - :code:`CommonNoLocalVolumeClient.test_downloader`: :func:`slack_transfer.functions.common.test_downloader`
        - :code:`CommonNoLocalVolumeClient.test_uploader`: :func:`slack_transfer.functions.common.test_uploader`
    """

    def get_file_volumes(
        self,
        channel_ids: Optional[Union[str, List[str]]] = None,
        auto_join: bool = False,
    ) -> float:
        """calculate rough volume for files which will be downloaded :func:`slack_transfer.functions.common.get_file_volumes`"""
        return get_file_volumes(
            client=self, channel_ids=channel_ids, auto_join=auto_join
        )

    def test_connection(self) -> None:
        """test general connection to slack using token :func:`slack_transfer.functions.common.test_connection`"""
        return test_connection(client=self)

    def test_downloader(self) -> None:
        """test the token as downloader mode :func:`slack_transfer.functions.common.test_downloader`"""
        return test_downloader(client=self)

    def test_uploader(self) -> None:
        """test the token as uploader mode :func:`slack_transfer.functions.common.test_uploader`"""
        return test_uploader(client=self)


class CommonClient(CommonNoLocalVolumeClient):
    """Common class among normal clients.
    Inherit :code:`slack_sdk.WebClient` via some classes.

    .. seealso::
        - https://github.com/slackapi/python-slack-sdk

    Args:
        local_data_dir (str): A directory you want to save and load your slack data.
        token (str; Optional): A string specifying an `xoxp-*` or `xoxb-*` token.
        base_url (str; Optional): A string representing the Slack API base URL.
            Default is `'https://www.slack.com/api/'`
        timeout (int; Optional): The maximum number of seconds the client will wait
            to connect and receive a response from Slack.
            Default is 30 seconds.
        ssl (SSLContext; Optional): An [`ssl.SSLContext`][1] instance, helpful for specifying
            your own custom certificate chain.
        proxy (str; Optional): String representing a fully-qualified URL to a proxy through
            which to route all requests to the Slack API. Even if this parameter
            is not specified, if any of the following environment variables are
            present, they will be loaded into this parameter: `HTTPS_PROXY`,
            `https_proxy`, `HTTP_PROXY` or `http_proxy`.
        headers (dict; Optional): Additional request headers to attach to all requests.

    .. seealso::
        - :code:`CommonClient.get_channels_list`: :func:`slack_transfer.functions.common.get_channels_list`
        - :code:`CommonClient.get_replies`: :func:`slack_transfer.functions.common.get_replies`
        - :code:`CommonClient.get_file_volumes`: :func:`slack_transfer.functions.common.get_file_volumes`
        - :code:`CommonClient.test_connection`: :func:`slack_transfer.functions.common.test_connection`
        - :code:`CommonClient.test_downloader`: :func:`slack_transfer.functions.common.test_downloader`
        - :code:`CommonClient.test_uploader`: :func:`slack_transfer.functions.common.test_uploader`
    """

    BASE_URL = "https://www.slack.com/api/"
    TYPE = "bot"

    def __init__(
        self,
        local_data_dir: str,
        token: Optional[str] = None,
        base_url: str = BASE_URL,
        timeout: int = 30,
        ssl: Optional[SSLContext] = None,
        proxy: Optional[str] = None,
        headers: Optional[dict] = None,
        user_agent_prefix: Optional[str] = None,
        user_agent_suffix: Optional[str] = None,
        team_id: Optional[str] = None,
        logger: Optional[logging.Logger] = None,
        retry_handlers: Optional[List[RetryHandler]] = None,
    ):
        if token is not None and self.TYPE == "bot" and not token.startswith("xoxb-"):
            raise ValueError("token have to be start with xoxb-")
        if token is not None and self.TYPE == "user" and not token.startswith("xoxp-"):
            raise ValueError("token have to be start with xoxp-")
        super().__init__(
            token=token,
            base_url=base_url,
            timeout=timeout,
            ssl=ssl,
            proxy=proxy,
            headers=headers,
            user_agent_prefix=user_agent_prefix,
            user_agent_suffix=user_agent_suffix,
            team_id=team_id,
            logger=logger,
            retry_handlers=retry_handlers,
        )
        self.local_data_dir: str = local_data_dir
        os.makedirs(os.path.join(self.local_data_dir, "files"), exist_ok=True)
        os.makedirs(os.path.join(self.local_data_dir, "channels"), exist_ok=True)
        os.makedirs(os.path.join(self.local_data_dir, "bookmarks"), exist_ok=True)
        os.makedirs(os.path.join(self.local_data_dir, "emojis"), exist_ok=True)


class UploaderClientABC(CommonClient):
    """Abstract class for Uploader Clients. This is the same as :class:`slack_transfer._base.CommonClient`.
    But, this is introduced for easily identify the client is uploader."""

    pass


class DownloaderClientABC(CommonClient):
    """Abstract class for Downloader Clients. This is the same as :class:`slack_transfer._base.CommonClient`.
    But, this is introduced for easily identify the client is downloader."""

    pass



import argparse
import math

from slack_transfer._base import CommonNoLocalVolumeClient


def set_parser_file_volume(parser: argparse.ArgumentParser) -> None:
    """See file_volume section in :doc:`../../../user_guide/cli`"""
    parser.add_argument(
        "--token",
        type=str,
        required=True,
        help="Token obtained from slack. Starts with `xoxb-`",
    )
    parser.add_argument(
        "--channel_ids",
        type=str,
        default=None,
        help="channel ids you want to check the total volume of files. If not set, set to all available channels. "
        + "Set by comma-separation for multiple inputs",
    )
    parser.add_argument(
        "--auto_join",
        action="store_true",
        help="if bot is not in channel automatically join when this flag is used.",
    )


def main_file_volume(args: argparse.Namespace) -> None:
    """See file_volume section in :doc:`../../../user_guide/cli`"""
    client = CommonNoLocalVolumeClient(token=args.token)
    volume = client.get_file_volumes(
        channel_ids=(
            args.channel_ids.split(",") if args.channel_ids is not None else None
        )
    )
    units = ["B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB"]
    i_units = math.floor(math.log(volume, 1024)) if volume > 0 else 0
    size = round(volume / 1024**i_units, 2)

    print(f"{size} {units[i_units]} (Actual download size is usually smaller.)")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    set_parser_file_volume(parser=parser)
    args = parser.parse_args()
    main_file_volume(args=args)



from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple
from typing import Union

import tqdm

from ._base import DownloaderClientABC
from ._base import UploaderClientABC
from .functions.download import download_bookmark
from .functions.download import download_channel_history
from .functions.download import download_channels_list
from .functions.download import download_emoji
from .functions.download import download_file
from .functions.download import download_members_list
from .functions.upload import check_channel_exists
from .functions.upload import check_insert_finished
from .functions.upload import check_upload_conflict
from .functions.upload import create_all_channels
from .functions.upload import data_insert
from .functions.upload import insert_bookmarks
from .functions.upload import upload_file


class DownloaderClient(DownloaderClientABC):
    """Downloader Client class.
    Inherit :code:`slack_sdk.WebClient` via some classes.

    .. seealso::
        - https://github.com/slackapi/python-slack-sdk

    Args:
        local_data_dir (str): A directory you want to save and load your slack data.
        token (str; Optional): A string specifying an `xoxp-*` or `xoxb-*` token.
        base_url (str; Optional): A string representing the Slack API base URL.
            Default is `'https://www.slack.com/api/'`
        timeout (int; Optional): The maximum number of seconds the client will wait
            to connect and receive a response from Slack.
            Default is 30 seconds.
        ssl (SSLContext; Optional): An [`ssl.SSLContext`][1] instance, helpful for specifying
            your own custom certificate chain.
        proxy (str; Optional): String representing a fully-qualified URL to a proxy through
            which to route all requests to the Slack API. Even if this parameter
            is not specified, if any of the following environment variables are
            present, they will be loaded into this parameter: `HTTPS_PROXY`,
            `https_proxy`, `HTTP_PROXY` or `http_proxy`.
        headers (dict; Optional): Additional request headers to attach to all requests.

    .. seealso::
        - :code:`DownloaderClient.get_channels_list`: :func:`slack_transfer.functions.common.get_channels_list`
        - :code:`DownloaderClient.get_replies`: :func:`slack_transfer.functions.common.get_replies`
        - :code:`DownloaderClient.get_file_volumes`: :func:`slack_transfer.functions.common.get_file_volumes`
        - :code:`DownloaderClient.test_connection`: :func:`slack_transfer.functions.common.test_connection`
        - :code:`DownloaderClient.test_downloader`: :func:`slack_transfer.functions.common.test_downloader`
        - :code:`DownloaderClient.test_uploader`: :func:`slack_transfer.functions.common.test_uploader`
        - :code:`DownloaderClient.download_channels_list`: :func:`slack_transfer.functions.download.download_channels_list`
        - :code:`DownloaderClient.download_file`: :func:`slack_transfer.functions.download.download_file`
        - :code:`DownloaderClient.download_channel_history`: :func:`slack_transfer.functions.download.download_channel_history`
        - :code:`DownloaderClient.download_members_list`: :func:`slack_transfer.functions.download.download_members_list`
        - :code:`DownloaderClient.download_bookmark`: :func:`slack_transfer.functions.download.download_bookmark`
        - :code:`DownloaderClient.download_emoji`: :func:`slack_transfer.functions.download.download_emoji`
    """

    def download_channels_list(self) -> List[Dict]:
        return download_channels_list(client=self)

    def download_file(self, file_id: str, file_name: str, url_private: str) -> None:
        return download_file(
            client=self, file_id=file_id, file_name=file_name, url_private=url_private
        )

    def download_channel_history(
        self,
        channel_id: str,
        channel_name: str,
        latest: Optional[str] = None,
        ts_progress_bar: Optional[tqdm.tqdm] = None,
        ts_now: Optional[int] = None,
        auto_join: bool = True,
    ) -> None:
        return download_channel_history(
            client=self,
            channel_id=channel_id,
            channel_name=channel_name,
            latest=latest,
            ts_progress_bar=ts_progress_bar,
            ts_now=ts_now,
            auto_join=auto_join,
        )

    def download_members_list(self) -> List[Dict]:
        return download_members_list(client=self)

    def download_bookmark(
        self, channel_id: str, channel_name: str, auto_join: bool = True
    ) -> None:
        return download_bookmark(
            client=self,
            channel_id=channel_id,
            channel_name=channel_name,
            auto_join=auto_join,
        )

    def download_emoji(self) -> None:
        return download_emoji(client=self)


class UploaderClient(UploaderClientABC):
    """Uploader Client class.
    Inherit :code:`slack_sdk.WebClient` via some classes.

    .. seealso::
        - https://github.com/slackapi/python-slack-sdk

    Args:
        local_data_dir (str): A directory you want to save and load your slack data.
        token (str; Optional): A string specifying an `xoxp-*` or `xoxb-*` token.
        base_url (str; Optional): A string representing the Slack API base URL.
            Default is `'https://www.slack.com/api/'`
        timeout (int; Optional): The maximum number of seconds the client will wait
            to connect and receive a response from Slack.
            Default is 30 seconds.
        ssl (SSLContext; Optional): An [`ssl.SSLContext`][1] instance, helpful for specifying
            your own custom certificate chain.
        proxy (str; Optional): String representing a fully-qualified URL to a proxy through
            which to route all requests to the Slack API. Even if this parameter
            is not specified, if any of the following environment variables are
            present, they will be loaded into this parameter: `HTTPS_PROXY`,
            `https_proxy`, `HTTP_PROXY` or `http_proxy`.
        headers (dict; Optional): Additional request headers to attach to all requests.

    .. seealso::
        - :code:`DownloaderClient.get_channels_list`: :func:`slack_transfer.functions.common.get_channels_list`
        - :code:`DownloaderClient.get_replies`: :func:`slack_transfer.functions.common.get_replies`
        - :code:`DownloaderClient.get_file_volumes`: :func:`slack_transfer.functions.common.get_file_volumes`
        - :code:`DownloaderClient.test_connection`: :func:`slack_transfer.functions.common.test_connection`
        - :code:`DownloaderClient.test_downloader`: :func:`slack_transfer.functions.common.test_downloader`
        - :code:`DownloaderClient.test_uploader`: :func:`slack_transfer.functions.common.test_uploader`
        - :code:`DownloaderClient.create_all_channels`: :func:`slack_transfer.functions.upload.create_all_channels`
        - :code:`DownloaderClient.upload_file`: :func:`slack_transfer.functions.upload.upload_file`
        - :code:`DownloaderClient.data_insert`: :func:`slack_transfer.functions.upload.data_insert`
        - :code:`DownloaderClient.check_upload_conflict`: :func:`slack_transfer.functions.upload.check_upload_conflict`
        - :code:`DownloaderClient.insert_bookmarks`: :func:`slack_transfer.functions.upload.insert_bookmarks`
        - :code:`DownloaderClient.check_insert_finished`: :func:`slack_transfer.functions.upload.check_insert_finished`
        - :code:`DownloaderClient.check_channel_exists`: :func:`slack_transfer.functions.upload.check_channel_exists`
    """

    def create_all_channels(
        self,
        channel_names: Optional[List[str]] = None,
        name_mappings: Optional[Dict[str, str]] = None,
    ) -> None:
        return create_all_channels(
            client=self, channel_names=channel_names, name_mappings=name_mappings
        )

    def upload_file(
        self,
        old_file_id: str,
        file_name: str,
        channel_id: Optional[str] = None,
        title: Optional[str] = None,
        filetype: Optional[str] = None,
        thread_ts: Optional[str] = None,
    ) -> Optional[Tuple[str, str]]:

        return upload_file(
            client=self,
            old_file_id=old_file_id,
            file_name=file_name,
            channel_id=channel_id,
            title=title,
            filetype=filetype,
            thread_ts=thread_ts,
        )

    def data_insert(
        self,
        channel_name: str,
        old_members_dict: Dict[str, str],
        old_members_icon_url_dict: Optional[Dict[str, str]] = None,
        old_channel_name: Optional[str] = None,
        time_zone: str = "Asia/Tokyo",
        progress: Union[bool, tqdm.tqdm] = True,
    ) -> str:
        return data_insert(
            client=self,
            channel_name=channel_name,
            old_members_dict=old_members_dict,
            old_members_icon_url_dict=old_members_icon_url_dict,
            old_channel_name=old_channel_name,
            time_zone=time_zone,
            progress=progress,
        )

    def check_upload_conflict(
        self, name_mappings: Optional[Dict[str, str]] = None
    ) -> List[str]:
        return check_upload_conflict(client=self, name_mappings=name_mappings)

    def insert_bookmarks(self, channel_id: str, old_channel_name: str = None) -> None:
        return insert_bookmarks(
            client=self, channel_id=channel_id, old_channel_name=old_channel_name
        )

    def check_insert_finished(
        self,
        channel_name: str,
        old_members_dict: Dict[str, str],
        old_channel_name: Optional[str] = None,
        time_zone: str = "Asia/Tokyo",
    ) -> bool:
        return check_insert_finished(
            client=self,
            channel_name=channel_name,
            old_members_dict=old_members_dict,
            old_channel_name=old_channel_name,
            time_zone=time_zone,
        )

    def check_channel_exists(self, channel_name: str) -> bool:
        return check_channel_exists(client=self, channel_name=channel_name)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['slack_transfer', 'slack_transfer.cli', 'slack_transfer.functions']

package_data = \
{'': ['*']}

install_requires = \
['markdownify>=0.11.2,<0.12.0',
 'prompt-toolkit>=3.0.30,<4.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'requests>=2.28.1,<3.0.0',
 'slack-sdk>=3.18.1,<4.0.0',
 'tqdm>=4.64.0,<5.0.0']

entry_points = \
{'console_scripts': ['slack_transfer = slack_transfer.cli.main:main']}

setup_kwargs = {
    'name': 'slack-transfer',
    'version': '0.2.8',
    'description': '',
    'long_description': '# slack_transfer\n[![python](https://img.shields.io/pypi/pyversions/slack-transfer.svg)](https://pypi.org/project/slack-transfer)\n[![pypi](https://img.shields.io/pypi/v/slack-transfer.svg)](https://pypi.org/project/slack-transfer)\n[![CI](https://github.com/masanorihirano/slack_transfer/actions/workflows/ci.yml/badge.svg)](https://github.com/masanorihirano/slack_transfer/actions/workflows/ci.yml)\n[![downloads](https://img.shields.io/pypi/dm/slack-transfer)](https://pypi.org/project/slack-transfer)\n[![code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/d8c6e7691ae4462592be32394699b09c)](https://www.codacy.com/gh/masanorihirano/slack_transfer/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=masanorihirano/slack_transfer&amp;utm_campaign=Badge_Grade)\n\n## Documentations & User Guides\nslack_transfer is a tool for transferring messages to the other slack workspace.\n\nDocumentations are available on [readthedoc](https://slack-transfer.readthedocs.io).\n-   [User guide (en)](https://slack-transfer.readthedocs.io/en/latest/user_guide/index.html)\n-   [ユーザーガイド (日本語)](https://slack-transfer.readthedocs.io/en/latest/user_guide/index_ja.html)\n\n## Install\nThis package is available on pypi as [`slack-transfer`](https://pypi.org/project/slack-transfer/)\n```bash\n$ pip install slack-transfer\n$ python\n>> import slack_transfer\n```\nPlease note that you have to user `slack_transfer` instead of `slack-transfer` for importing.\n\n[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/masanorihirano/slack_transfer/blob/main/examples/slack_transfer.ipynb)\n\n## Issues and Contribution\nAbout issues (bugs):\n-   You can report issues [here](https://github.com/masanorihirano/slack_transfer/issues).\n-   日本語でissueを立てて構いません．\n-   There are no guarantee to support or fix those issues.\n\nContributions:\n-   You can send pull requests (PRs) to this repository.\n-   But, there are no guarantee to merge your PRs.\n',
    'author': 'Masanori HIRANO',
    'author_email': 'masa.hirano.1996@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<3.11',
}


setup(**setup_kwargs)
